%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Play Pong with the nScope
% Channel A is used to control the paddle
% Voltage must be between 0V and 5V
% NDM 9/19/2014
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% clear memory and the screen
clc
clear
close all

disp('nScope Pong!');
disp('Press any key to exit');

% load the lib
javaaddpath(fullfile(pwd,'nu_hidapi-1.1.jar'));
import com.codeminders.hidapi.*;

if ClassPathLibraryLoader.loadNativeHIDLibrary()
    mgr = HIDManager.getInstance();
    devs = mgr.listDevices();
else
    disp('Failed to load HID Library');
    return;
end

% get the nscope from the HID devices
connected = 0;
tried = 0;
while(connected == 0 && tried < 20)
    j = -1;
    for i=1:length(devs)
        if strcmp(devs(i).getProduct_string(),'nScope')
            j = i;
        end
    end
    
    if (j>=0)
        pic = devs(j).open();
        pause(0.01);
        if isempty(pic)
           tried = tried+1; % not really needed anymore, just in case
        else
            pic.enableBlocking();
            connected = 1;
        end 
    end
end

if (connected == 0)
    disp('  Fail - could not connect');
    return;
elseif (connected == 1)
    disp('  Connected to nScope');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% send an initialization request
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ComType = 7; % init and request firmware version
PacketID = 0; % didn't get any packets yet
% everything else 0

% build the init packet
output_buffer = zeros(1,65);
% first byte always 0
output_buffer(2) = bitor(ComType,bitshift(PacketID,3)); % rightshift is negative, leftshift is positive
% can double check binary values with dec2bin

% send the packet
pic.write(output_buffer);

% read the response
input_buffer = typecast(pic.matlabRead(),'uint8'); % grab an incoming packet
PacketID = bitand(input_buffer(1),7);
PacketID = 0; % ignore that first PacketID, its bad
% nothing else interesting in this packet

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% set the channel sensitivities, sample rate, and start continuous sampling
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% build the desired packet
ComType = 1;
output_buffer = zeros(1,65);
% first byte always = 0
output_buffer(2) = bitor(ComType,bitshift(PacketID,3));
output_buffer(3) = bitor(bitor(2,bitshift(1,2)),bitshift(1,5));
output_buffer(4) = bitand(10,255);
output_buffer(5) = 0; 
output_buffer(6) = 0; 
output_buffer(7) = 0; 
output_buffer(8) = 0; 
output_buffer(9) = 0; 
output_buffer(10) = 0;

% send the packet
pic.write(output_buffer);

% read the response
input_buffer = typecast(pic.matlabRead(),'uint8'); % grab an incoming packet
PacketID = bitand(input_buffer(1),7);
% nothing else interesting in this packet

disp('  Collecting at 200Hz...');

figure();
ballx = 100;
bally = randi([1,4]);
balldir = 1;
dy = (rand - 0.5)/4;
score = 0;
paddle = 2.5;
prevpaddle = 2.5;
scoretime = 0;


trace = zeros(1,400);
gotDataPoints = 0;

% read packets and reply with PacketID 
% end at keypress or after 3 minutes
tic;
while((toc < 40) && (score < 25))
    % build a request packet
	output_buffer = zeros(1,65);
	output_buffer(2) = bitor(ComType,bitshift(PacketID,3));
	
	% send the packet
	pic.write(output_buffer);
    
    % read the response
    input_buffer = zeros(1,65);
	input_buffer = typecast(pic.matlabRead(),'uint8'); % grab an incoming packet
	PacketID = bitand(input_buffer(1),7);
	Pwr = bitand(bitshift(input_buffer(1),-3),3);
	SamplesSent = bitor(bitand(bitshift(input_buffer(1),-5),7),bitshift(bitand(input_buffer(2),3),3));
	PowerUsage = bitand(bitshift(input_buffer(2),2),63);
    if SamplesSent == 31 % this means the nscope has updated its properties
        SamplesSent = 0; 
    end
       
	% FrequencyCounter = input_buffer(3 and 4) % don't need
	for(i=1:double(SamplesSent))
        a = uint16(input_buffer(3+i*3)); % -3 on all the index to account for i starting at 1
        b = uint16(input_buffer(2+i*3));
        dataA(i) = double(bitor(bitshift(bitand(a,3),8),b)); 
	end
    
    gotDataPoints = gotDataPoints + double(SamplesSent);      
    
    if (double(SamplesSent)>0)
        trace = [trace(1+double(SamplesSent):length(trace)),dataA(1:double(SamplesSent))];
        
        paddle = ((1023 - trace(399))/1023-.5)*10;
        ballx = ballx + balldir*3;
        bally = bally + dy;

        % ball escapes the paddle
        if (ballx > 390)
            ballx = 100;
            bally = randi([1,4]);
            balldir = 1;
            dy = (rand - 0.5)/4;
            score = score - 10;
        end

        % is the ball on the paddle
        if((ballx > 350)&&(ballx < 400)&&(bally > paddle-.5)&&(bally < paddle + .5))
            if toc > scoretime + 1
                score = score + 5;
                scoretime = toc;
                balldir = balldir * -1;
                if paddle > prevpaddle
                    dy = paddle - prevpaddle;
                    if dy > 2
                        dy = 2;
                    end
                    if dy < -2
                        dy = -2;
                    end
                end
            end
        end

        % switch direction if hit back wall
        if ((ballx < 10)&&(balldir == -1))
            balldir = balldir * -1;
        end

        % switch direction if hit the vertical
        if bally > 4.5
            dy = dy * -1;
        end
        if bally < 0.5
            dy = dy * -1;
        end

        prevpaddle = paddle;
        
        clf;
        hold on;
        plot(1:length(trace),((1023 - trace)/1023-.5)*10,'k:');
        plot(ballx,bally,'.','MarkerSize',72);
        plot(399,paddle,'s','MarkerSize',72);
        text(20,5,['Score = ' num2str(score)]);
        t = toc;
        text(20,4.75,['Time = ' num2str(t)]);
        
        axis([0 length(trace) -0.5 5.5]);
        pause(0.001);
    end
end

if (score >= 25)
    text(200,2.5,'You win!');
else
	text(200,2.5,'Try again!');
end

disp('  Collection Complete!');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% send packet to stop sampling
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ComType = 1;
Rq = 3; % stop sampling
output_buffer = zeros(1,65);
output_buffer(2) = bitor(ComType,bitshift(PacketID,3));
output_buffer(3) = Rq;

% send the packet
pic.write(output_buffer);

% read the response
input_buffer = typecast(pic.matlabRead(),'uint8'); % grab an incoming packet
PacketID = bitand(input_buffer(1),7);
Pwr = bitand(bitshift(input_buffer(1),-3),3);
SamplesSent = bitor(bitand(bitshift(input_buffer(1),-5),3),bitshift(bitand(input_buffer(2),3),3));
PowerUsage = bitand(bitshift(input_buffer(2),-2),63);
% nothing else interesting in this packet

% close the port and free the library
pic.close();
mgr.release();

disp('Disconnected from nScope');